package dev.fyloz.colorrecipesexplorer.service

import com.nhaarman.mockitokotlin2.*
import dev.fyloz.colorrecipesexplorer.model.*
import dev.fyloz.colorrecipesexplorer.repository.MixMaterialRepository
import org.junit.jupiter.api.Test
import kotlin.test.assertEquals
import kotlin.test.assertFalse
import kotlin.test.assertNotEquals
import kotlin.test.assertTrue

class MixMaterialServiceTest : AbstractModelServiceTest<MixMaterial, MixMaterialService, MixMaterialRepository>() {
    override val repository: MixMaterialRepository = mock()
    private val materialService: MaterialService = mock()
    override val service: MixMaterialService = spy(MixMaterialServiceImpl(repository, materialService))

    private val material: Material = material(id = 0L)
    override val entity: MixMaterial = mixMaterial(id = 0L, material = material, quantity = 1000f)
    override val anotherEntity: MixMaterial = mixMaterial(id = 1L, material = material)

    // existsByMaterial()

    @Test
    fun `existsByMaterial() returns true when a mix material with the given material exists`() {
        whenever(repository.existsByMaterial(material)).doReturn(true)

        val found = service.existsByMaterial(material)

        assertTrue(found)
    }

    @Test
    fun `existsByMaterial() returns false when no mix material with the given material exists`() {
        whenever(repository.existsByMaterial(material)).doReturn(false)

        val found = service.existsByMaterial(material)

        assertFalse(found)
    }

    // create()

    @Test
    fun `create(set) calls create() for each MixMaterialDto`() {
        val mixMaterialDtos = setOf(
            mixMaterialDto(materialId = 0L, quantity = 1000f, position = 1),
            mixMaterialDto(materialId = 1L, quantity = 2000f, position = 2),
            mixMaterialDto(materialId = 2L, quantity = 3000f, position = 3),
            mixMaterialDto(materialId = 3L, quantity = 4000f, position = 4)
        )

        doAnswer {
            with(it.arguments[0] as MixMaterialDto) {
                mixMaterial(
                    material = material(id = this.materialId),
                    quantity = this.quantity,
                    position = this.position
                )
            }
        }.whenever(service).create(any<MixMaterialDto>())

        val found = service.create(mixMaterialDtos)

        mixMaterialDtos.forEach { dto ->
            verify(service).create(dto)
            assertTrue {
                found.any {
                    it.material.id == dto.materialId && it.quantity == dto.quantity && it.position == dto.position
                }
            }
        }
    }

    @Test
    fun `create() creates a mix material according to the given MixUpdateDto`() {
        val mixMaterialDto = mixMaterialDto(materialId = 0L, quantity = 1000f, position = 1)

        whenever(materialService.getById(mixMaterialDto.materialId)).doAnswer { material(id = it.arguments[0] as Long) }

        val found = service.create(mixMaterialDto)

        assertTrue {
            found.material.id == mixMaterialDto.materialId &&
                    found.quantity == mixMaterialDto.quantity &&
                    found.position == mixMaterialDto.position
        }
    }

    // updateQuantity()

    @Test
    fun `updateQuantity() updates the given mix material with the given quantity`() {
        val quantity = 5000f
        assertNotEquals(quantity, entity.quantity, message = "Quantities must not be equals for this test to works")

        doAnswer { it.arguments[0] }.whenever(service).update(any())

        val found = service.updateQuantity(entity, quantity)

        assertEquals(found.quantity, quantity)
    }
}
